<?php

namespace Drupal\mailjet;

use Mailjet\Resources;

/**
 * Implement MailjetHandler interface.
 */
class MailjetHandler implements MailjetHandlerInterface {

  /**
   * Mailjet client.
   *
   * @var \Mailjet\Client
   */
  protected $mailjetClient;

  /**
   * {@inheritdoc}
   *
   * @return array|false|null
   *   Return array with result or false.
   */
  public function __construct(MailjetFactory $mailjetClient) {
    $this->mailjetClient = $mailjetClient->create();
  }

  /**
   * {@inheritdoc}
   *
   * @return array|false|null
   *   Return array with result or false.
   */
  public function getMailjetContactLists($limit = 0): ?array {
    $filters = [
      'Limit' => $limit,
      'Sort' => 'Name ASC',
    ];
    $response = $this->mailjetClient->get(Resources::$Contactslist, ['filters' => $filters]);

    if ($response->success()) {
      return $response->getData();
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   *
   * @return array|false|null
   *   Return array with result or false.
   */
  public function getMailjetContactListByName($name): ?array {
    $filters = [
      'Name' => $name,
    ];
    $response = $this->mailjetClient->get(Resources::$Contactslist, ['filters' => $filters]);

    if ($response->success() && $response->getCount() > 0) {
      return $response->getData();
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   *
   * @return array|false|null
   *   Return array with result or false.
   */
  public function getMailjetContactProperties($limit = 0): ?array {
    $filters = [
      'Limit' => $limit,
    ];
    $response = $this->mailjetClient->get(Resources::$Contactmetadata, ['filters' => $filters]);

    if ($response->success()) {
      return $response->getData();
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   *
   * @return array|false|null
   *   Return array with result or false.
   */
  public function createMailjetContactList($listName): ?array {
    if (empty($listName)) {
      return FALSE;
    }

    $body = [
      'Name' => $listName,
    ];
    $response = $this->mailjetClient->post(Resources::$Contactslist, ['body' => $body]);

    if ($response->success()) {
      return $response->getData();
    }

    return FALSE;
  }

  /**
   * Get contact properties.
   *
   * @return array|false|null
   *   Return array with response or false.
   */
  public function getContactProperties(): ?array {
    $filters = [
      'limit' => 0,
      'Sort' => 'Name ASC',
    ];
    $response = $this->mailjetClient->get(Resources::$Contactmetadata, ['filters' => $filters]);

    if ($response->success()) {
      return $response->getData();
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   *
   * @return array|false|null
   *   Return array with result or false.
   */
  public function createMailjetContactProperty($name, $type = "str"): ?array {
    if (empty($name)) {
      return FALSE;
    }

    // Name: the name of the custom data field.
    // DataType: the type of data that is being stored
    // (this can be either a str, int, float or bool).
    // NameSpace: this can be either static or historic.
    $body = [
      'Datatype' => $type,
      'Name' => $name,
      'NameSpace' => "static",
    ];
    $response = $this->mailjetClient->post(Resources::$Contactmetadata, ['body' => $body]);

    if ($response->success()) {
      return $response->getData();
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   *
   * @return array|false|null
   *   Return array with result or false.
   */
  public function updateMailjetContactProperty($id, $name, $type = "str"): ?array {
    if (empty($name)) {
      return FALSE;
    }

    // Name: the name of the custom data field.
    // DataType: the type of data that is being stored
    // (this can be either a str, int, float or bool).
    // NameSpace: this can be either static or historic.
    $body = [
      'Datatype' => $type,
      'Name' => $name,
      'NameSpace' => "static",
    ];
    $response = $this->mailjetClient->put(Resources::$Contactmetadata,
      ['id' => $id, 'body' => $body]);

    if ($response->success()) {
      return $response->getData();
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   *
   * @return array|false|null
   *   Return array with result or false.
   */
  public function syncMailjetContacts($contactListId, array $contacts, $action = 'addforce'): ?array {
    $body = [
      'Action' => $action,
      'Contacts' => $contacts,
    ];
    $response = $this->mailjetClient->post(Resources::$ContactslistManagemanycontacts,
      ['id' => $contactListId, 'body' => $body]);

    if ($response->success()) {
      return $response->getData();
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   *
   * @return array|false|null
   *   Return array with result or false.
   */
  public function syncMailjetContact($contactListId, array $contact, $action = 'addforce'): ?array {
    $body = [
      'Action' => $action,
      'Email' => $contact['Email'],
    ];

    if (!empty($contact['Properties'])) {
      $body['Properties'] = $contact['Properties'];
    }

    $response = $this->mailjetClient->post(Resources::$ContactslistManagecontact,
      ['id' => $contactListId, 'body' => $body]);

    if ($response->success()) {
      return $response->getData();
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   *
   * @return array|false|null
   *   Return array with result or false.
   */
  public function createApiToken(array $params): ?array {
    $response = $this->mailjetClient->post(Resources::$Apitoken, ['body' => $params]);
    if ($response->success()) {
      return $response->getData();
    }

    return FALSE;
  }

}
